# -*- coding: utf-8 -*-
__author__ = 'Dener Rodrigues'

import re
from datetime import datetime
from sigpdv.tools.log import traced
from sigpdv.models.cad_clientes_data import CaixaCadClientes
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.models.cad_finalizadora_data import CaixaCadFinalizadora
from sigpdv.tools.config import config
from sigpdv.models.db_utils import ConectarApi
from sigpdv.control.utils import remove_accents, converte_tz
import time

server = config['db_ges_host']
username = config['db_ges_user']
pwd = config['db_ges_pass']
dbname = config['db_ges_name']
port = config['db_ges_port']
dbemp = config['db_ges_emp']

sup_server = config['erp_sup_host']
sup_dbname = config['erp_sup_name']
sup_port = config['erp_sup_port']
sup_pwd = config['erp_sup_pass']
sup_username= config['erp_sup_user']

uid = None


class ErrConnRPC(Exception):
    """Base"""


@traced
def get_state_pedido_dav(numero_pedido):
    """
    Retorna o state do pedido do erp
    """
    api_sup = ConectarApi('supervisor', sup_server, sup_port, sup_username, sup_pwd, sup_dbname)
    api_erp = ConectarApi('estoquepedidos', server, port, username, pwd, dbname)
    args = [('numero_pedido', '=', 'PVE-%010d' % int(numero_pedido))]
    if dbemp:
        emp = dbemp.replace(' ', '').replace('-', ',').split(',')
        emp_list = []
        for x in emp:
            emp_list.append(int(x))
        emp_list = list(set(emp_list))
    else:
        res = api_sup.execute('ctrl_sup_get_emrpesa')
        emp_list = res

    res = api_erp.execute('ctrl_estped_get_emrpesa', {'empresa': emp_list[0]})
    epc_company_id = int(res.get('retorno')[0])
    args.append(('company_id', '=', epc_company_id))
    res = api_erp.execute('ctrl_estped_get_pedidos', {'args': str(args),
                                                      'campos': str(['state', 'exportado_frente']),
                                                      'empresa': epc_company_id})
    pedido_data = res.get('retorno')
    if not pedido_data:
        return 'I', True

    return pedido_data[0].get('state'), pedido_data[0].get('exportado_frente')


@traced
def cancelar_pedidos(nro_pedidos=[]):
    cancelado = False
    api_sup = ConectarApi('supervisor', sup_server, sup_port, sup_username, sup_pwd, sup_dbname)
    if not nro_pedidos:
        res = api_sup.execute('ctrl_sup_get_davs',
                              {'args': str([('supdva_restricao', 'in', (0, 1))]),
                               'campos': str(['supdva_nro_pedido'])})
        if res.get('retorno'):
            nro_pedidos = map(lambda x: 'PVE-%010d' % x.get('supdva_nro_pedido'), res.get('retorno'))

    if nro_pedidos:
        api_erp = ConectarApi('estoquepedidos', server, port, username, pwd, dbname)
        if dbemp:
            emp = dbemp.replace(' ', '').replace('-', ',').split(',')
            emp_list = []
            for x in emp:
                emp_list.append(int(x))
            emp_list = list(set(emp_list))
        else:
            res = api_sup.execute('ctrl_sup_get_emrpesa')
            emp_list = res

        res = api_erp.execute('ctrl_estped_get_emrpesa', {'empresa': emp_list[0]})
        epc_company_id = int(res.get('retorno')[0])
        res = api_erp.execute('ctrl_estped_get_pedidos', {'args': str([('numero_pedido', 'in', nro_pedidos),
                                                                       ('state', '=', 'C'),
                                                                       ('company_id', '=', epc_company_id)]),
                                                          'campos': str(['numero_pedido']),
                                                          'empresa': epc_company_id})
        numero_dav_cancelado = []
        for pedido in res.get('retorno'):
            numero_dav_cancelado.append(pedido.get('numero_pedido').split('PVE-')[1])
            cancelado = True

        if numero_dav_cancelado:
            api_sup.execute('ctrl_sup_atualiza_dav', {'numero_davs': str(numero_dav_cancelado),
                                                      'vals': str({'supdva_restricao': -1})
                                                      })
    return cancelado


@traced
def fatura_pedido_dav(numero_dav, loja='', caixa='', coo=''):
    api_erp = ConectarApi('estoquepedidos', server, port, username, pwd, dbname)

    if dbemp:
        emp = dbemp.replace(' ', '').replace('-', ',').split(',')
        emp_list = []
        for x in emp:
            emp_list.append(int(x))
        emp_list = list(set(emp_list))
    else:
        api_sup = ConectarApi('supervisor', sup_server, sup_port, sup_username, sup_pwd, sup_dbname)
        res = api_sup.execute('ctrl_sup_get_emrpesa')
        emp_list = res

    res = api_erp.execute('ctrl_estped_get_emrpesa', {'empresa': emp_list[0]})
    epc_company_id = int(res.get('retorno')[0])
    res = api_erp.execute('ctrl_estped_get_pedidos', {'args': str([('numero_pedido', '=', 'PVE-%010d' % int(numero_dav)),
                                                                   ('company_id', '=', epc_company_id)]),
                                                      'campos': str(['state']),
                                                      'empresa': epc_company_id})
    pedido_dados = res.get('retorno')[0]
    if pedido_dados.get('state') == 'F':  # o pedido ja foi faturado no erp
        return True

    vals = {'state': 'F', 'loja': '%d' % loja, 'caixa': '%d' % caixa, 'coo': '%d' % coo, 'exportado_frente': True}
    api_erp.execute('ctrl_estped_atualiza_pedidos', {'pedido_ids': str([pedido_dados.get('id')]),
                                                     'vals': str(vals)})
    # o pedido pode ter mais de um titulo, deletar todos eles, pois estao em previsao.
    api_erp.execute('ctrl_estped_deleta_titulos', {'pedido_id': pedido_dados.get('id')})
    return True


@traced
def faturar_pedidos_pendentes(dav, loja='', caixa='', coo=''):
    res = []
    try:
        f = fatura_pedido_dav(dav.supdva_nro_pedido, loja, caixa, coo)
        if not f:
            raise Exception('Pedido nao faturado')
    except Exception as e:
        traced.error('DAV{0:010d} (N)'.format(dav.supdva_nro_pedido) + '\n' + e)
        res.append('DAV{0:010d}'.format(dav.supdva_nro_pedido))
    return res


def import_dav(parent=None):
    def get_cpf_cnpj_cliente(api_erp, pessoa_id):
        """
        Obtem o documento do cliente informado (no gestão)
        retorna o documento com 15 caracteres e 0 a esquerda
        """
        res = api_erp.execute('ctrl_estped_get_cpf', {'pessoa_id': pessoa_id})
        doc = '0'
        if res.get('retorno'):
            doc = res.get('retorno')[0] or '0'

        return re.sub('[^0-9]', '', doc).rjust(15, '0')

    def get_qtda_emb(api_erp, p_embalagem_id):
        """
        Obtem a quantidade da embalem informada
        retorna int
        """
        res = {'qtda': 1,
               'sigla': 'UN'}
        if isinstance(p_embalagem_id, list):
            p_embalagem_id = p_embalagem_id[0]

        if not p_embalagem_id:
            return res

        res_api = api_erp.execute('ctrl_estped_get_qtd_embalagem',
                                  {'emb_id': p_embalagem_id,
                                   'campos': str(['embalagem_id'])})
        res_api = res_api.get('retorno')[0]
        res.update(res_api)
        return res

    def get_codigo_barra(api_erp, item, emp_list):
        """
        Obtem o codigo de barra do item informado (no gestão)
        retorna o codigo de barra com 14 caracteres e 0 a esquerda
        """
        qtde_pedida = item.get('qtde_pedida')
        grade_id = item.get('grade_id')[0]
        emb_id = item.get('produto_embalagem_id')[0]
        print 'GET COD BARRA', grade_id, emb_id
        args = [('grade_id', '=', int(grade_id)),
                ('active', '=', True),
                ('company_id', '=', int(emp_list[0]))]
        campos = ['codigo_barra', 'produto_embalagem_id']
        res = api_erp.execute('ctrl_estped_get_acb', {'args': str(args),
                                                      'campos': str(campos),
                                                      'empresa': emp_list[0]})
        codigo_barra = []
        menor_qtd = 0
        for i in res.get('retorno'):
            qtd = get_qtda_emb(api_erp, i.get('produto_embalagem_id'))['qtda']
            menor_qtd = not menor_qtd and qtd or menor_qtd
            if i.get('produto_embalagem_id') == emb_id:
                codigo_barra = (qtd, i.get('codigo_barra'))
                break

            if qtd <= menor_qtd and qtde_pedida % qtd == 0:
                menor_qtd = qtd
                codigo_barra = (qtd, i.get('codigo_barra'))

        return codigo_barra

    # -------------------------------------------- // -------------------------------------------- #

    api_erp = ConectarApi('estoquepedidos', server, port, username, pwd, dbname)
    api_sup = ConectarApi('supervisor', sup_server, sup_port, sup_username, sup_pwd, sup_dbname)
    # BUSCAR OS PEDIDOS QUE NAO ESTAO EM ESBOCO E AINDA NAO FORAM ENVIADOS
    args = [('state', 'in', ['V', 'C', 'F', 'D', 'O']),
            ('tipo', '=', 'V'),
            ('exportado_frente', '=', False)]
    if dbemp:
        emp = dbemp.replace(' ', '').replace('-', ',').split(',')
        emp_list = []
        for x in emp:
            emp_list.append(int(x))
        emp_list = list(set(emp_list))
    else:
        res = api_sup.execute('ctrl_sup_get_emrpesa')
        emp_list = res

    res = api_erp.execute('ctrl_estped_get_emrpesa', {'empresa': emp_list[0]})
    epc_company_id = int(res.get('retorno')[0])
    args.append(('company_id', '=', epc_company_id))
    campos_pedido = ['create_date', 'numero_pedido', 'pessoa_id', 'vendedor_id', 'vr_desconto',
                     'valor_total', 'company_id', 'state']
    res = api_erp.execute('ctrl_estped_get_pedidos', {'args': str(args),
                                                      'campos': str(campos_pedido),
                                                      'empresa': epc_company_id})
    pedido_data = res.get('retorno')
    err_dict = {}
    res = api_sup.execute('ctrl_sup_get_davs', {'args': str([('supdva_restricao', '=', 1)]),
                                                'campos': str(['supdva_nro_pedido'])})
    if res.get('retorno'):
        nro_pedidos = map(lambda x: 'PVE-%010d' % x.get('supdva_nro_pedido'), res.get('retorno'))
        args = [('state', '=', 'R'), ('tipo', '=', 'V'),
                ('numero_pedido', 'in', nro_pedidos),
                ('company_id', '=', epc_company_id)]
        res = api_erp.execute('ctrl_estped_get_pedidos', {'args': str(args),
                                                          'campos': str(campos_pedido),
                                                          'empresa': epc_company_id})
        pedido_data.extend(res.get('retorno'))

    if not pedido_data:
        return err_dict

    # OBTER OS DADOS DOS PEDIDOS ENCONTRADOS
    # READ PEDIDOS
    parent.show_info_proccess(True, 'Aguarde! Processando %s pedido(s)...' % (len(pedido_data)))
    parent.QtGui.QApplication.processEvents()
    parent.QtGui.QApplication.processEvents() # quando chama este metodo apenas uma vez, a mensagem fica com a tela preta
    time.sleep(0.5)
    pedidos_enviados_ids = []  # pedidos que nao sao para aparecer mais no frente(faturados e cancelados no ERP)
    list_pedidos_ids = []
    dict_dav = {}
    codigo_vendedores = []
    pedido_ids = []
    num_pedidos = []
    for i in pedido_data:
        num_pedidos.append(int(i.get('numero_pedido').split('PVE-')[1]))
        pedido_ids.append(i.get('id'))
        if i.get('vendedor_id'):
            codigo_vendedores.append(i.get('vendedor_id')[0])

    res = api_sup.execute('ctrl_sup_get_davs', {'args': str([('supdva_nro_pedido', 'in', num_pedidos)]),
                                                'campos': str(['supdva_nro_pedido', 'supdva_nro_cupom'])})
    map(lambda x: dict_dav.update({x.get('supdva_nro_pedido'): x}), res.get('retorno'))
    campos_item = ['produto_id', 'grade_id', 'produto_embalagem_id', 'vr_unitario',
                   'valor_total', 'vr_desconto', 'qtde_pedida', 'vendedor_id', 'pedido_id']
    res = api_erp.execute('ctrl_estped_get_itens',
                          {'args': str([('pedido_id', 'in', pedido_ids)]),
                           'campos': str(campos_item)})
    dados_itens = {}
    if res.get('retorno'):
        dados_itens = res.get('retorno')[0]
        for i, itens in dados_itens.items():
            for item in itens:
                if item.get('vendedor_id'):
                    codigo_vendedores.append(item.get('vendedor_id')[0])

    dict_vendedores = {}
    if codigo_vendedores:
        res = api_sup.execute('ctrl_sup_get_vendedor',
                              {'args': str([('supven_codigo', 'in', codigo_vendedores)]),
                               'campos': str(['supven_codigo'])})
        dict_vendedores = res.get('retorno')[0]

    # pedidos enviados com sucesso para o Supervisor
    caixa = config['caixa']
    for pedido in pedido_data:
        pedido_id = pedido.get('id')
        nro_pedido = int(re.sub('[^0-9]', '', pedido.get('numero_pedido')))
        loja = pedido.get('company_id') or None
        if loja and isinstance(loja, list):
            loja = loja[0]
        data_hora = pedido.get('create_date')
        juros = 0.0
        descontos = pedido.get('vr_desconto')
        restricao = 1
        total_dav = pedido.get('valor_total')
        err_dict.update({nro_pedido: []})

        # *** CHECAR CLIENTE DO PEDIDO *** #
        dav_super = dict_dav.get(int(pedido.get('numero_pedido').split('PVE-')[1]))

        # pedidos que foram faturados ou cancelados no erp e nao estao no supervisor
        if not dav_super:
            if pedido.get('state') in ['F', 'C']:
                pedidos_enviados_ids.append(pedido.get('id'))
                continue

        else: # pedido que foi modificado e já está no supervisor
            vals_dav = {}
            if pedido.get('state') == 'F': # pedidos que foram faturados no erp
                vals_dav['supdva_restricao'] = 2
                pedidos_enviados_ids.append(pedido.get('id'))

            elif pedido.get('state') == 'C': # pedidos que foram cancelados no erp
                vals_dav['supdva_restricao'] = -1
                pedidos_enviados_ids.append(pedido.get('id'))
            if vals_dav:
                vals_dav['id'] = dav_super.get('id')
                api_sup.execute('ctrl_sup_write_dav', {'vals': str(vals_dav)})
                continue

            if dav_super.get('supdva_nro_cupom'):  # Verifica se já foi emitido o cupom p/ o pedido
                err_dict[nro_pedido] += ['Pedido já foi emitido']
                pedidos_enviados_ids.append(pedido.get('id'))
                continue

            api_sup.execute('ctrl_sup_deletar_dav', {'id': dav_super.get('id')})

        cliente = pedido.get('pessoa_id') and pedido.get('pessoa_id')[0]
        nome_cliente = remove_accents(pedido.get('pessoa_id') and pedido.get('pessoa_id')[1] or '')
        cnpj_cpf_cliente = cliente and get_cpf_cnpj_cliente(api_erp, cliente) or ''
        cliente = False
        if cnpj_cpf_cliente:
            cliente = CaixaCadClientes.get_by(supcli_cnpj_cpf=cnpj_cpf_cliente)
        if not cliente and cnpj_cpf_cliente:
            err_dict[nro_pedido] += [
                'Cliente {0:>s} nao esta cadastrado no Frente'.format(nome_cliente)]
            continue

        if pedido.get('state') == 'R': # pedidos em esboço sao deletados e nao sao criados no sup.
            continue

        nome_cliente = cliente and cliente.name or ''
        cpf_cnpj_cliente = cliente and cliente.supcli_cnpj_cpf or ''

        # CHECAR VENDEDOR DO PEDIDO SE É OPERADOR DO FRENTE
        vendedor = vendedor_id = pedido.get('vendedor_id') or None
        if vendedor and isinstance(vendedor, list):
            vendedor_id = vendedor[0]
            nome_vendedor = remove_accents(vendedor[1])
            cad_vendedor = dict_vendedores.get(str(vendedor_id))
            if cad_vendedor:
                vendedor_id = cad_vendedor.get('id')
            else:
                err_dict[nro_pedido] += [
                    'Vendedor %s-%s nao esta cadastrado no Supervisor' % (vendedor_id,
                                                                          nome_vendedor)]
                continue

        # *** OBTER OS DADOS DOS ITENS *** #
        if not dados_itens.get(str(pedido_id)):
            err_dict[nro_pedido] += ['Sem itens lancados']
            continue

        sequence_item = 1
        items_list = []
        for item in dados_itens.get(str(pedido_id)):
            vendedor_item = vendedor_item_id = item.get('vendedor_id') or None
            if vendedor_item and isinstance(vendedor_item, list):
                vendedor_item_id = vendedor_item[0]
                nome_vendedor = remove_accents(vendedor_item[1])
                cad_vendedor = dict_vendedores.get(str(vendedor_item_id))
                if cad_vendedor:
                    vendedor_item_id = cad_vendedor.get('id')
                else:
                    err_dict[nro_pedido] += [
                        'Vendedor %s-%s nao esta cadastrado no Supervisor' % (vendedor_item_id,
                                                                              nome_vendedor)]
                    continue

            traced.info('dav id ' + str(pedido_id))
            traced.info('Nro. Pedido: ' + str(nro_pedido))
            item_descricao = remove_accents(item.get('produto_id')[1])
            codigo_barra = get_codigo_barra(api_erp, item, emp_list)
            traced.info('EAN ' + str(codigo_barra))
            if not codigo_barra:
                err_dict[nro_pedido] += [
                    'Produto ({0:>s}) sem codigo de barras no SigERP '
                    'para \n\tGrade: "{1:>s}" e \n\tEmbalagem: "{2:>s}"'.format(
                        item_descricao,
                        item.get('grade_id')[1],
                        item.get('produto_embalagem_id')[1])]
                break

            traced.info('cbarra ' + str(codigo_barra[1]))
            codigo_barra_e = CaixaCadItens.get_by(supite_codigo_item=codigo_barra[1],
                                                  supite_loja=parent.loja.supcfc_loja)
            if not codigo_barra_e:
                err_dict[nro_pedido] += [
                    'Produto ({0:>s}) sem codigo de barras no Frente'.format(
                         item_descricao)]
                break

            qtd_emb_orig = get_qtda_emb(api_erp, item.get('produto_embalagem_id'))
            traced.info(qtd_emb_orig)
            traced.info('emb ' + str(qtd_emb_orig) + str(codigo_barra))
            vorig = item.get('vr_unitario') / qtd_emb_orig['qtda'] * qtd_emb_orig['qtda']
            vajus = item.get('vr_unitario') / qtd_emb_orig['qtda'] * codigo_barra[0]
            qorig = (item.get('qtde_pedida') * qtd_emb_orig['qtda']) / qtd_emb_orig['qtda']
            qajus = (item.get('qtde_pedida') * qtd_emb_orig['qtda']) / codigo_barra[0]

            traced.info(str(vorig * qorig) + '|' + str(vorig) + '-' + str(qorig))
            traced.info(str(vajus * qajus) + '|' + str(vajus) + '-'  + str(qajus))

            items_list.append(dict(
                supdvo_sequencia=sequence_item,
                supdvo_numero_do_dav=nro_pedido,
                supdvo_data_hora=data_hora,
                supdvo_item=codigo_barra[1],
                supdvo_quantidade=qajus,
                supdvo_quantidade_conferida=item.get('qtde_atendida') or 0,
                supdvo_vr_total=item.get('valor_total'),
                supdvo_desconto_item=item.get('vr_desconto') or 0,
                supdvo_acrescimo_item=item.get('vr_acrescimo') or 0,
                supdvo_venda_cancelamento='V',
                supdvo_descricao=item_descricao.split(" - ")[1],
                supdvo_unidade=qtd_emb_orig.get('sigla'),
                supdvo_vr_unit=vajus,
                supdvo_totalizador_parcial='',
            ))
            if vendedor_item_id:
                items_list[-1]['supdvo_vendedor_id'] = vendedor_item_id
            sequence_item += 1

        if err_dict[nro_pedido]:
            continue

        # *** OBTER OS DADOS DOS FECHAMENTOS *** #
        res = api_erp.execute('ctrl_estped_get_fechamentos',
                              {'args': str([('capa_id', '=', int(pedido_id))]),
                               'campos': str([])})
        if not res.get('retorno'):
            err_dict[nro_pedido] += ['Sem fechamentos lançado']
            continue

        fechamentos_data = res.get('retorno')
        sequence_fechamento = 1
        fechamento_list = []
        sem_finalizadora_frente = False
        for fechamento in fechamentos_data:
            res = api_erp.execute('ctrl_estped_get_cond_forma',
                            {'condicao_forma_id': fechamento['condicao_forma_id'][0]})
            cod_ecf = res.get('retorno')[0].get('finalizadora_na_ecf')
            finalizadora = CaixaCadFinalizadora.get_by(supfin_codigo=cod_ecf)
            if not finalizadora:
                err_dict[nro_pedido] += [
                    'Condição de pagamento: %d, não cadastrado no Frente' % cod_ecf]
                sem_finalizadora_frente = True
                break

            fechamento_list.append(dict(
                id=fechamento.get('id'),
                supdvf_cheque_agencia=fechamento.get('pedcdc_chq_agencia') or 0,
                supdvf_cheque_banco=fechamento.get('pedcdc_chq_banco') or 0,
                supdvf_cheque_cnpj=fechamento.get('pedcdc_chq_emitente') or '',
                supdvf_cheque_conta=int(re.sub('[^0-9]', '',
                                               (str(fechamento.get('pedcdc_chq_conta') or '0')))),
                supdvf_cheque_numero=int(re.sub('[^0-9]', '',
                                               (str(fechamento.get('pedcdc_chq_numero') or '0')))),
                supdvf_sequencia=sequence_fechamento,
                supdvf_finalizadora=cod_ecf,
                supdvf_valor=fechamento['valor']
            ))
            sequence_fechamento += 1

        if sem_finalizadora_frente:
            continue

        # SE TUDO DEU CERTO VAMOS LANÇAR O PEDIDO COMO DAV
        create_date = converte_tz(False, datetime.now(), 'America/Sao_Paulo', 'UTC')
        vals_capa = dict(id=pedido_id,
                          supdva_nro_pedido=nro_pedido,
                          supdva_data_hora=data_hora,
                          supdva_cliente=cpf_cnpj_cliente,
                          supdva_nome_cliente=nome_cliente,
                          supdva_juros=juros,
                          supdva_descontos=descontos,
                          supdva_loja=loja,
                          supdva_caixa=caixa,
                          supdva_restricao=restricao,
                          supdva_total_dav=total_dav)
        if vendedor_id:
            vals_capa['supdva_vendedor_id']=vendedor_id

        for fechamento in fechamento_list:
            id_erp = fechamento.get('id')
            fechamento.pop('id')
            # vencimentos do fechamento
            res = api_erp.execute('ctrl_estped_get_parcelas',
                                  {'args': str([('pedido_condicao_id', '=', int(id_erp))]),
                                   'campos': str([])})
            if not res.get('retorno'):
                err_dict[nro_pedido] += ['Sem vencimento lançado']
                continue

            vencimentos_data = res.get('retorno')
            sequence_vencimento = 1
            fechamento['vencimentos'] = []
            for vencimento in vencimentos_data:
                fechamento['vencimentos'].append(
                                  dict(supdvv_seq_vencimento=sequence_vencimento,
                                       supdvv_valor_parcela=vencimento.get('vr_parcela'),
                                       supdvv_vencimento=vencimento.get('data_vencimento')
                                       )
                                  )
                sequence_vencimento += 1

        res = api_sup.execute('ctrl_sup_criar_dav', {'vals_capa': str(vals_capa),
                                               'itens': str(items_list),
                                               'fechamentos': str(fechamento_list)})
        list_pedidos_ids.append(pedido_id)

    if list_pedidos_ids:
        api_erp.execute('ctrl_estped_atualiza_pedidos', {'pedido_ids': str(list_pedidos_ids),
                                                         'vals': str({'exportado_frente': True,
                                                                      'loja': config['loja'],
                                                                      'caixa': caixa}
                                                                     )
                                                         })

    # grava como enviado os pedidos que nao sao mais para aparecer na tela de dav do frente
    if pedidos_enviados_ids:
        api_erp.execute('ctrl_estped_atualiza_pedidos', {'pedido_ids': str(pedidos_enviados_ids),
                                                         'vals': str({'exportado_frente': True})
                                                         })

    parent.show_info_proccess(False)
    return err_dict

