#!/usr/bin/env python
# -*- Mode: Python; coding: utf-8 -*-
from sqlalchemy.exc import OperationalError

import sys
from PySide2 import QtCore, QtGui
from PySide2.QtWidgets import QDialog, QDoubleSpinBox, QLineEdit, QComboBox, QDateEdit, \
    QMessageBox, QTableWidget, QListWidget, QApplication
from ui_login import Ui_Dialog
from messagebox import message_dlg, MSG_INFO
from sigpdv.tools.config import config
from sigpdv.models.supervisor import SupervisorCadOperadores

K1 = ord('1') if not config['usarcodificado'] else config['bt_num_1']
K2 = ord('2') if not config['usarcodificado'] else config['bt_num_2']
K3 = ord('3') if not config['usarcodificado'] else config['bt_num_3']
K4 = ord('4') if not config['usarcodificado'] else config['bt_num_4']
K5 = ord('5') if not config['usarcodificado'] else config['bt_num_5']
K6 = ord('6') if not config['usarcodificado'] else config['bt_num_6']
K7 = ord('7') if not config['usarcodificado'] else config['bt_num_7']
K8 = ord('8') if not config['usarcodificado'] else config['bt_num_8']
K8 = ord('8') if not config['usarcodificado'] else config['bt_num_8']
K9 = ord('9') if not config['usarcodificado'] else config['bt_num_9']
K0 = ord('0') if not config['usarcodificado'] else config['bt_num_0']
KDICT = {
    K1: '1',
    K2: '2',
    K3: '3',
    K4: '4',
    K5: '5',
    K6: '6',
    K7: '7',
    K8: '8',
    K9: '9',
    K0: '0'
}


class LoginForm(QDialog):
    def __init__(self, parent=None, db='C', superuser=False):
        QDialog.__init__(self, parent, QtCore.Qt.Dialog | QtCore.Qt.FramelessWindowHint)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        self.setModal(True)
        self.estacionamento = False
        self.teclado = False
        self.btn_cancel_info_dialog = False
        self.cancel_pinpad = False

        self._tbl_operadores = SupervisorCadOperadores

        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        try:
            # self._tbl_operadores.get_by(supope_codigo=1)
            pass
        except Exception as e:
            from sigpdv.dav_pve.pve import exception_message_box

            exception_message_box(str(e))
            sys.exit()
        self.parent = parent
        self.retorno = False
        # SIGNALS
        self._CONTROL_FOCUS = self.ui.editUser
        self.ui.editUser.setFocus()
        # KeysNumPad
        self.ui.btnKEY_0.clicked.connect(self.KEY_0_Click)
        self.ui.btnKEY_1.clicked.connect(self.KEY_1_Click)
        self.ui.btnKEY_2.clicked.connect(self.KEY_2_Click)
        self.ui.btnKEY_3.clicked.connect(self.KEY_3_Click)
        self.ui.btnKEY_4.clicked.connect(self.KEY_4_Click)
        self.ui.btnKEY_5.clicked.connect(self.KEY_5_Click)
        self.ui.btnKEY_6.clicked.connect(self.KEY_6_Click)
        self.ui.btnKEY_7.clicked.connect(self.KEY_7_Click)
        self.ui.btnKEY_8.clicked.connect(self.KEY_8_Click)
        self.ui.btnKEY_9.clicked.connect(self.KEY_9_Click)
        self.ui.btnKEY_RETURN.clicked.connect(self.KEY_RETURN_Click)
        self.ui.btnKEY_BACKSPACE.clicked.connect(self.KEY_BACKSPACE_Click)

    def setControlFocus(self, old, new, is_escape=False):
        old_name, new_name = ('', '')
        if old:
            old_name = old.objectName()
        if new:
            new_name = new.objectName()
        if not is_escape:
            self._tab_order.append(new)
        if old:
            old.setStyleSheet("")

        self._CONTROL_FOCUS = new

        if type(new) in [QLineEdit, QDoubleSpinBox, QComboBox, QDateEdit]:
            new.setStyleSheet("background-color: rgba(153, 204, 204, 255); color: rgb(0, 0, 127)")

            if type(old) in [QDoubleSpinBox, QDateEdit]:
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Up,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Down,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_End,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)

            elif type(new) in [QDoubleSpinBox, QDateEdit]:
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Up,
                                     QtCore.Qt.KeyboardModifiers())
                new.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Down,
                                     QtCore.Qt.KeyboardModifiers())
                new.keyPressEvent(ev)
        elif type(new) == QTableWidget:
            new.setStyleSheet('background-color: rgb(255, 252, 205)')
        try:
            new.selectAll()
            new.setFocus()
        except Exception as e:
            pass

        return True

    def _on_key_click(self, key):
        self._tempo_ocioso = 0
        text = chr(key) if key < 256 else ''
        text = text if text != '-' else ''
        ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, key, QtCore.Qt.KeyboardModifiers(), text=text)
        obj_name = str(self._CONTROL_FOCUS.objectName()).lower()
        if type(self._CONTROL_FOCUS) in [QLineEdit, QDoubleSpinBox, QDateEdit,
                                         QComboBox] and key == QtCore.Qt.Key_Return:
            try:
                return eval('self.%s()' % obj_name)
            except Exception as err:

                self.keyPressEvent(ev)

        # BACKSPACE
        if type(self._CONTROL_FOCUS) in [QLineEdit, QDoubleSpinBox, QDateEdit] and \
                        key == QtCore.Qt.Key_Backspace.__int__():
            self._CONTROL_FOCUS.selectAll()
            self._CONTROL_FOCUS.clear()
            self._CONTROL_FOCUS.selectAll()
        if type(self._CONTROL_FOCUS) in [QLineEdit, QDoubleSpinBox, QDateEdit] and \
                        key == QtCore.Qt.Key_Down.__int__():
            return

        self._CONTROL_FOCUS.keyPressEvent(ev)
        if type(self._CONTROL_FOCUS) == QListWidget:
            self._CONTROL_FOCUS.clearFocus()

    def get_key_name(self, key):
        try:
            name = key.name[4:]
        except Exception as e:
            return ''

        if name == 'Comma':
            name = ','
        if name == 'Period':
            name = '.'
        return name

    def keyPressEvent(self, ev):
        if not ev.key():
            return ''

        if self._CONTROL_FOCUS in [self.ui.editUser, self.ui.editPwd]:
            ks = [K0, K1, K2, K3, K4, K5, K6, K7, K8, K9]
            if ev.key() in ks:
                self._CONTROL_FOCUS.keyPressEvent(ev)
                return self.get_key_name(ev.key())
            if not config.get('usarcodificado'):
                if ev.key() not in ks:
                    if ev.key() in [QtCore.Qt.Key_Enter.__int__(), QtCore.Qt.Key_Return.__int__()]:
                        self.KEY_RETURN_Click()
                    else:
                        self._CONTROL_FOCUS.keyPressEvent(ev)
                    return
            else:
                if ev.key() == config['bt_entrar']:
                    self.KEY_RETURN_Click()
                    return
            if ev.key() in [QtCore.Qt.Key_Enter.__int__(), QtCore.Qt.Key_Return.__int__()]:
                self.KEY_RETURN_Click()
                return

        return self.get_key_name(ev.key())

    def KEY_0_Click(self):
        return self._on_key_click(QtCore.Qt.Key_0)

    def KEY_1_Click(self):
        return self._on_key_click(QtCore.Qt.Key_1)

    def KEY_2_Click(self):
        return self._on_key_click(QtCore.Qt.Key_2)

    def KEY_3_Click(self):
        return self._on_key_click(QtCore.Qt.Key_3)

    def KEY_4_Click(self):
        return self._on_key_click(QtCore.Qt.Key_4)

    def KEY_5_Click(self):
        return self._on_key_click(QtCore.Qt.Key_5)

    def KEY_6_Click(self):
        return self._on_key_click(QtCore.Qt.Key_6)

    def KEY_7_Click(self):
        return self._on_key_click(QtCore.Qt.Key_7)

    def KEY_8_Click(self):
        return self._on_key_click(QtCore.Qt.Key_8)

    def KEY_9_Click(self):
        return self._on_key_click(QtCore.Qt.Key_9)

    def KEY_BACKSPACE_Click(self):
        return self._on_key_click(QtCore.Qt.Key_Backspace)

    def KEY_RETURN_Click(self):
        if self._CONTROL_FOCUS == self.ui.editUser:
            self._CONTROL_FOCUS = self.ui.editPwd
        else:
            self._CONTROL_FOCUS = self.ui.editUser
        return self.accept()

    def accept(self):
        str_user = self.ui.editUser.text()
        str_pwd = self.ui.editPwd.text()
        if not str_user:
            QMessageBox.information(self, 'Login',
                                          self.tr('Digite seu código de usuário.'),
                                    QMessageBox.Ok)
            self.ui.editUser.setFocus()
        elif not str_pwd:
            self.ui.editPwd.setFocus()
        else:
            try:
                operador = self._tbl_operadores.get_by(supope_codigo=int(str_user), active=True)
                if not operador:
                    message_dlg(self, MSG_INFO,
                                '<font color="red"><h2>Usuário não cadastrado.</h2></font>')
                    self.ui.editUser.setFocus()
                    self.ui.editUser.clear()
                    self.ui.editPwd.clear()
                else:
                    pwd = operador.supope_senha
                    user = operador.name
                    if pwd != str_pwd:
                        message_dlg(self, MSG_INFO,
                                    '<font color="red"><h2>Senha incorreta!</h2></font>')
                        self.ui.editPwd.clear()
                        self.ui.editPwd.setFocus()
                    else:
                        self.retorno = operador
                        super(LoginForm, self).accept()
            except OperationalError as e:
                QMessageBox.information(self, 'Login',
                                        self.tr('Sem conexao com o Banco de Dados Supervisor.'),
                                        QMessageBox.Ok)


def start():
    app = QApplication(sys.argv)
    form = LoginForm(db='S')
    form.show()
    app.exec_()

if __name__ == "__main__":
    try:
        start()
    except:
        sys.exit()
